
/*
 This sample shows how to implement a custom data source.
 For simplicity, we will just make a wrapper for standard file operations.
*/

#include <stdio.h>
#include <stdlib.h>
#include "..\..\..\bmp.h"
#include "..\..\..\j2k-codec.h"

/* Define callback functions */

int  __stdcall MySeek(void *data_source, int offset)
{
 return -1; /* Our pipe won't be seekable */

 /* For seekable pipe we should write: */
 return (fseek((FILE*)data_source, offset, SEEK_SET)==0 ? 0 : -1);
}

int  __stdcall MyRead(void *ptr, int size, void *data_source)
{
 return fread(ptr, 1, size, (FILE*)data_source);
}

void __stdcall MyClose(void *data_source)
{
 if(data_source) fclose((FILE*)data_source);
}

/* ------------------------------------------------------------------- */

int main(int ArgC, char **ArgV)
{
 void *j2k_image; 
 char  Key[] = "Your registration key";
 int   j2k_ver, j2k_err, width, height, components, resolutions=0;  
 unsigned char *buffer=0; int size, bpp;

 FILE *f; J2K_Callbacks cbs; 

 /* Get the DLL version and make sure it is at least 1.1 */

 j2k_ver = J2K_getVersion(); if(j2k_ver < 0x110000) return -1;

 /* Check that the user specified the file name */

 if(ArgC<2){ printf("\n Please, specify the J2K image file name!\n\n"); system("pause"); return -1; }

 /* Remove the demo version restrictions */

 J2K_Unlock(Key); 

 /* Prepare callbacks structure */

 cbs.seek  = MySeek;
 cbs.read  = MyRead;
 cbs.close = MyClose;

 /* Open J2K file */

 f = fopen(ArgV[1], "rb"); if(!f){ printf("\n File '%s' opening error.\n", ArgV[1]); return -1; }

 /* Open custom data source */

 j2k_image = J2K_OpenCustom((void*)f, &cbs); 
 if(!j2k_image){ printf("\n J2K_OpenCustom() error: %s.\n", J2K_getLastErrorStr()); return -1; }

 /* Let's get some info */

 j2k_err = J2K_GetInfo(j2k_image, &width, &height, &components);
 if(j2k_err != J2KERR_SUCCESS)
 {
  printf("\n J2K_GetInfo() error: %s.\n", J2K_getErrorStr(j2k_err)); 
  J2K_Close(j2k_image);
  return -1; 
 }

 printf("\n Image info: %dx%d, %d component(s).\n", width, height, components);

 j2k_err = J2K_GetResolutions(j2k_image, &resolutions);
 if(j2k_err != J2KERR_SUCCESS) printf("\n J2K_GetResolutions() error: %s.\n", J2K_getErrorStr(j2k_err));

 printf("\n %d resolution(s) available.\n", resolutions);

 /* Decode the image */

 j2k_err = J2K_Decode(j2k_image, &buffer, &size, "", NULL);
 if(j2k_err != J2KERR_SUCCESS) printf("\n J2K_Decode() error: %s.\n", J2K_getErrorStr(j2k_err));
 else
 {
  /* Finally save the decoded image as BMP file */

  bpp = (components==1) ? 1 : 4;

  SaveAsBMP("decoded.bmp", buffer, width, height, bpp);

  printf("\n Done!\n");
 }

 /* Clean up */

 J2K_Close(j2k_image); 

 return 0;
}
